from shutil import get_terminal_size

from pytest import CaptureFixture, FixtureRequest, Parser, fixture, hookimpl


# ---------------------------------- printer --------------------------------- #
class NormalPrinter:
    option = [
        (
            ("--kasima-skip",),
            {"action": "store_true", "help": "disable pytest-kasima output"},
        )
    ]

    print = print

    @staticmethod
    def rule(t: str = ""):
        _pad = "-"
        _term_width = get_terminal_size()[0]  # ターミナルの幅
        print(f"\n{t:{_pad}^{_term_width}}")


printer: type[NormalPrinter] = NormalPrinter

try:

    class RichPrinter(NormalPrinter):
        option = NormalPrinter.option + [
            (
                ("--kasima-rich-skip",),
                {"action": "store_true", "help": "disable rich output"},
            )
        ]

        from rich import print
        from rich.rule import Rule as _Rule

        @staticmethod
        def rule(t: str = ""):
            RichPrinter.print(RichPrinter._Rule(title=t))

    printer = RichPrinter

except ImportError:
    pass
# ---------------------------------------------------------------------------- #


def pytest_addoption(parser: Parser):
    """オプション追加"""
    group = parser.getgroup("kasima")
    for a, k in printer.option:
        group.addoption(*a, **k)


@fixture()
def pretty_print(request: FixtureRequest):
    """printの出力を見やすくする"""
    global printer

    p = (
        NormalPrinter
        if request.config.getoption("--kasima-rich-skip", default=False)
        else printer
    )

    def _print(*a, **k):
        print()  # OPTIMIZE: ファイル名 の横に出力されるから改行
        p.rule(f" {request.node.name} ")
        p.print(*a, **k)
        p.rule()
        print()

    yield _print


@fixture(autouse=True)
def printcap(request: FixtureRequest, capsys: CaptureFixture, pretty_print):
    """captureされた出力をpretty_print"""

    yield

    if not request.config.getoption("--kasima-skip"):
        with capsys.disabled():
            pretty_print(capsys.readouterr().out)


"""
@hookimpl(trylast=True)
def pytest_configure(config):
    pass
"""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_kasima']

package_data = \
{'': ['*']}

install_requires = \
['pytest>=7.2.1,<8.0.0']

extras_require = \
{'rich': ['rich']}

entry_points = \
{'pytest11': ['kasima = pytest_kasima']}

setup_kwargs = {
    'name': 'pytest-kasima',
    'version': '0.1.5',
    'description': 'Display horizontal lines above and below the captured standard output for easy viewing.',
    'long_description': "[![PyPI version](https://badge.fury.io/py/pytest-kasima.svg)](https://badge.fury.io/py/pytest-kasima)\n[![Python Versions](https://img.shields.io/pypi/pyversions/pytest-kasima.svg)](https://pypi.org/project/pytest-kasima)\n\n# pytest-kasima\n\n![normal](https://github.com/k4sima/pytest-kasima/blob/main/screenshots/normal.png?raw=true)\n\n## install\n\n```\npip install pytest-kasima\n```\n\nIf you're a fan of [rich library](https://github.com/Textualize/rich), can integrate\n\n![with_rich](https://github.com/k4sima/pytest-kasima/blob/main/screenshots/rich.png?raw=true)\n\n```\npip install pytest-kasima[rich]\n```\n\n## usage\n\nDisplay horizontal lines above and below the captured standard output for easy viewing.\n\n`--capture=no` or `-s` option is not required.\n\n### options\n\n- `--kasima-skip` - If you don't like this plugin, disable it :smiley:\n- `--kasima-rich-skip` - Skip the rich integration\n",
    'author': 'k4sima',
    'author_email': '44926913+k4sima@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/k4sima/pytest-kasima',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
